'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _enzyme = require('enzyme');

var _chai = require('chai');

var _sinon = require('sinon');

var _reactMotion = require('react-motion');

var _SwipeableViews = require('../src/SwipeableViews');

var _SwipeableViews2 = _interopRequireDefault(_SwipeableViews);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable flowtype/require-valid-file-annotation */
/* eslint-env mocha */

describe('SwipeableViews', function () {
  describe('props: children', function () {
    it('should render the children', function () {
      var wrapper = (0, _enzyme.mount)(_react2.default.createElement(
        _SwipeableViews2.default,
        null,
        _react2.default.createElement(
          'div',
          null,
          'slide n°1'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°2'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°3'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°4'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°5'
        )
      ));

      _chai.assert.strictEqual(wrapper.text(), 'slide n°1slide n°2slide n°3slide n°4slide n°5', 'Should render each slide.');
    });
  });

  describe('props: onTouchStart', function () {
    it('should trigger when we bind it', function () {
      var handleTouchStart = (0, _sinon.spy)();
      var wrapper = (0, _enzyme.mount)(_react2.default.createElement(
        _SwipeableViews2.default,
        { onTouchStart: handleTouchStart },
        _react2.default.createElement(
          'div',
          null,
          'slide n°1'
        )
      ));

      wrapper.simulate('touchStart', {
        touches: [{}]
      });
      _chai.assert.strictEqual(handleTouchStart.callCount, 1, 'Should be called');
    });

    it('should trigger when we disable the swipe', function () {
      var handleTouchStart = (0, _sinon.spy)();
      var wrapper = (0, _enzyme.mount)(_react2.default.createElement(
        _SwipeableViews2.default,
        { disabled: true, onTouchStart: handleTouchStart },
        _react2.default.createElement(
          'div',
          null,
          'slide n°1'
        )
      ));

      wrapper.simulate('touchStart', {
        touches: [{}]
      });
      _chai.assert.strictEqual(handleTouchStart.callCount, 1, 'Should be called');
    });
  });

  describe('props: onTouchEnd', function () {
    it('should trigger when we bind it', function () {
      var handleTouchEnd = (0, _sinon.spy)();
      var wrapper = (0, _enzyme.mount)(_react2.default.createElement(
        _SwipeableViews2.default,
        { onTouchEnd: handleTouchEnd },
        _react2.default.createElement(
          'div',
          null,
          'slide n°1'
        )
      ));

      wrapper.simulate('touchEnd');
      _chai.assert.strictEqual(handleTouchEnd.callCount, 1, 'Should be called');
    });
  });

  describe('props: animateTransitions', function () {
    it('should use a spring if animateTransitions is true', function () {
      var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(
        _SwipeableViews2.default,
        null,
        _react2.default.createElement(
          'div',
          null,
          'slide n°1'
        )
      ));

      _chai.assert.deepEqual(wrapper.find(_reactMotion.Motion).at(0).props().style, {
        translate: {
          damping: 30,
          precision: 0.01,
          stiffness: 300,
          val: 0
        },
        height: 0
      });
    });

    it('should not use a spring if animateTransitions is false', function () {
      var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(
        _SwipeableViews2.default,
        { animateTransitions: false },
        _react2.default.createElement(
          'div',
          null,
          'slide n°1'
        )
      ));

      _chai.assert.deepEqual(wrapper.find(_reactMotion.Motion).at(0).props().style, {
        translate: 0,
        height: 0
      });
    });
  });

  describe('swipe detection', function () {
    var instance = void 0;
    var wrapper = void 0;

    beforeEach(function () {
      wrapper = (0, _enzyme.mount)(_react2.default.createElement(
        _SwipeableViews2.default,
        null,
        _react2.default.createElement(
          'div',
          null,
          'slide n°1'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°2'
        )
      ));

      wrapper.simulate('touchStart', {
        touches: [{
          pageX: 50,
          pageY: 50
        }]
      });
      instance = wrapper.instance();
      instance.viewLength = 200;
    });

    it('should not detect a swipe when scrolling', function () {
      wrapper.simulate('touchMove', {
        touches: [{
          pageX: 50,
          pageY: 60
        }]
      });
      _chai.assert.strictEqual(instance.isSwiping, false, 'Should not detect a swipe');
    });

    it('should detect a swipe when doing a clear movement', function () {
      wrapper.simulate('touchMove', {
        touches: [{
          pageX: 60,
          pageY: 50
        }],
        preventDefault: function preventDefault() {}
      });
      _chai.assert.strictEqual(instance.isSwiping, true, 'Should detect a swipe');
    });

    it('should wait for a clear movement to detect a swipe', function () {
      wrapper.simulate('touchMove', {
        touches: [{
          pageX: 48,
          pageY: 50
        }]
      });
      _chai.assert.strictEqual(instance.isSwiping, undefined, 'We do not know yet');

      wrapper.simulate('touchMove', {
        touches: [{
          pageX: 50,
          pageY: 48
        }]
      });
      _chai.assert.strictEqual(instance.isSwiping, undefined, 'We do not know yet');

      wrapper.simulate('touchMove', {
        touches: [{
          pageX: 40,
          pageY: 50
        }],
        preventDefault: function preventDefault() {}
      });
      _chai.assert.strictEqual(instance.isSwiping, true, 'Should detect a swipe');
    });
  });

  describe('nested views', function () {
    var wrapperParent = void 0;
    var wrapperNester = void 0;

    beforeEach(function () {
      wrapperParent = (0, _enzyme.mount)(_react2.default.createElement(
        _SwipeableViews2.default,
        { index: 1 },
        _react2.default.createElement(
          'div',
          null,
          'slide n°1'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°2'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°3'
        )
      ));

      wrapperNester = (0, _enzyme.mount)(_react2.default.createElement(
        _SwipeableViews2.default,
        { index: 0 },
        _react2.default.createElement(
          'div',
          null,
          'slide n°4'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°5'
        ),
        _react2.default.createElement(
          'div',
          null,
          'slide n°6'
        )
      ));

      var touchStartEvent = {
        touches: [{
          pageX: 50,
          pageY: 0
        }]
      };

      wrapperNester.simulate('touchStart', touchStartEvent);
      wrapperParent.simulate('touchStart', touchStartEvent);

      var instance1 = wrapperParent.instance();
      instance1.viewLength = 200;

      var instance2 = wrapperNester.instance();
      instance2.viewLength = 200;
    });

    afterEach(function () {
      wrapperNester.simulate('touchEnd');
      wrapperParent.simulate('touchEnd');
    });

    it('only the nested swipe should respond to the touch', function () {
      var touchMoveEvent1 = {
        touches: [{
          pageX: 45,
          pageY: 0
        }]
      };

      wrapperNester.simulate('touchMove', touchMoveEvent1);
      wrapperParent.simulate('touchMove', touchMoveEvent1);

      var touchMoveEvent2 = {
        touches: [{
          pageX: 40,
          pageY: 0
        }]
      };

      wrapperNester.simulate('touchMove', touchMoveEvent2);
      wrapperParent.simulate('touchMove', touchMoveEvent2);

      _chai.assert.strictEqual(wrapperNester.state().indexCurrent, 0.025);
      _chai.assert.strictEqual(wrapperParent.state().indexCurrent, 1);
    });

    it('only the parent swipe should respond to the touch', function () {
      var touchMoveEvent1 = {
        touches: [{
          pageX: 55,
          pageY: 0
        }]
      };

      wrapperNester.simulate('touchMove', touchMoveEvent1);
      wrapperParent.simulate('touchMove', touchMoveEvent1);

      var touchMoveEvent2 = {
        touches: [{
          pageX: 60,
          pageY: 0
        }]
      };

      wrapperNester.simulate('touchMove', touchMoveEvent2);
      wrapperParent.simulate('touchMove', touchMoveEvent2);

      _chai.assert.strictEqual(wrapperNester.state().indexCurrent, 0);
      _chai.assert.strictEqual(wrapperParent.state().indexCurrent, 0.975);
    });
  });
});